/* cl    /W3 httpget.c
 * bcc32 -w  httpget.c
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#include <windows.h>

/* try to tell linker where WinSock library is */
#if defined(_MSC_VER)
#   pragma comment(lib,"wsock32.lib")
#elif defined(__BORLANDC__)
#   pragma comment(lib,"mswsock.lib")
#endif


#define MAX_PAGE    (1024*64)

char    PageBuffer[MAX_PAGE];

int GetAddr(const char* HostName, int Port, struct sockaddr* Result)
    {
    struct hostent*     Host;
    SOCKADDR_IN         Address;

    memset(Result, 0, sizeof(*Result));
    memset(&Address, 0, sizeof(Address));

    Host                = gethostbyname(HostName);
    if(Host != NULL)
        {
        Address.sin_family  = AF_INET;
        Address.sin_port    = htons((short)Port);
        memcpy(&Address.sin_addr, Host->h_addr_list[0], Host->h_length);
        memcpy(Result, &Address, sizeof(Address));
        }
    return Host != NULL;
    }

void ReadPage(const char* Host, const char* Page)
    {
    struct sockaddr     SockAddr;
    SOCKET              Socket;
    int                 Port        = 80; /* HTTP */

    if(GetAddr(Host, Port, &SockAddr))
        {
        int     Status;

        Socket = socket(AF_INET, SOCK_STREAM, 0);
        Status = connect(Socket, &SockAddr, sizeof(SockAddr));
        if(Status >= 0)
            {
            DWORD   StartTime, EndTime;
            char    Request[512];
            char*   Rover   = PageBuffer;
            int     Read;

            sprintf(Request, "GET %s HTTP/1.0\n\n", Page);
            send(Socket, Request, strlen(Request), 0);

            StartTime = GetTickCount();
            for(Read=0;Read < MAX_PAGE;)
                {
                int     ThisRead;

                ThisRead = recv(Socket, Rover, MAX_PAGE-Read, 0);
                if(ThisRead == SOCKET_ERROR || ThisRead == 0)
                    break;
                else
                    {
                    Read    += ThisRead;
                    Rover   += ThisRead;
                    }
                }
            EndTime = GetTickCount();
            printf("%d milliseconds to read %d-byte page\n",
                EndTime-StartTime, Read);
            closesocket(Socket);
            }
        else
            fprintf(stderr, "connect failed (%d)\n",
                WSAGetLastError());
        }
    else
        fprintf(stderr, "Can't map hostname '%s'\n", Host);
    }

int main(int argc, char** argv)
    {
    WSADATA         WsaData;

    /* Right # of arguments? */
    assert(argc == 3);

    WSAStartup(0x0001, &WsaData);

    ReadPage(argv[1], argv[2]);

    WSACleanup();
    }
